==> automattic-superset-0.0.20240815102147rc1.tar.gz: automattic-superset-0.0.20240815102147rc1/superset/utils/google_sheets.py <==
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"Google Sheets Export features"

import datetime
import os

import pandas as pd
from flask import current_app

from superset import feature_flag_manager

LIB_GSPREAD_AVAILABLE = False
try:
    import gspread

    LIB_GSPREAD_AVAILABLE = True
except ModuleNotFoundError:
    pass

if feature_flag_manager.is_feature_enabled("GOOGLE_SHEETS_EXPORT"):
    assert LIB_GSPREAD_AVAILABLE, 'GOOGLE_SHEETS_EXPORT: Missing package.'
    assert isinstance(
        current_app.config["GOOGLE_SHEETS_EXPORT_SERVICE_ACCOUNT_JSON_PATH"],
        str,
    ), 'GOOGLE_SHEETS_EXPORT: Required valid service-account json path.'
    assert os.path.exists(
        current_app.config["GOOGLE_SHEETS_EXPORT_SERVICE_ACCOUNT_JSON_PATH"]
    ), 'GOOGLE_SHEETS_EXPORT: Required valid service-account json path.'
    assert isinstance(
        current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"],
        dict,
    ), 'GOOGLE_SHEETS_EXPORT: Required share permissions.'
    if current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"].get('perm_type') == 'anyone':
        assert 'email_address' in \
            current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"] and \
            current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"]['email_address']\
            == None, \
            'GOOGLE_SHEETS_EXPORT: For perm_type == "anyone", email_address must be set \
            to None'
    else:
        assert {"email_address", "perm_type", "role"} <= current_app.config[
            "GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"
        ].keys()


def upload_df_to_new_sheet(name: str, df: pd.DataFrame) -> str:
    assert feature_flag_manager.is_feature_enabled("GOOGLE_SHEETS_EXPORT")
    gc = gspread.service_account(
        filename=current_app.config["GOOGLE_SHEETS_EXPORT_SERVICE_ACCOUNT_JSON_PATH"],
    )
    spreadsheet = gc.create(f"{name} {datetime.datetime.utcnow().isoformat()}")
    spreadsheet.sheet1.update(
        range_name="A1",
        values=([df.columns.values.tolist()] + df.values.tolist()),
    )
    spreadsheet.share(**current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"])
    return spreadsheet.id

==> automattic-superset-0.0.20240816060926rc1.tar.gz: automattic-superset-0.0.20240816060926rc1/superset/utils/google_sheets.py <==
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"Google Sheets Export features"

import datetime
import os

import pandas as pd
from flask import current_app

from superset import feature_flag_manager

LIB_GSPREAD_AVAILABLE = False
try:
    import gspread

    LIB_GSPREAD_AVAILABLE = True
except ModuleNotFoundError:
    pass

if feature_flag_manager.is_feature_enabled("GOOGLE_SHEETS_EXPORT"):
    assert LIB_GSPREAD_AVAILABLE, 'GOOGLE_SHEETS_EXPORT: Missing package.'
    assert isinstance(
        current_app.config["GOOGLE_SHEETS_EXPORT_SERVICE_ACCOUNT_JSON_PATH"],
        str,
    ), 'GOOGLE_SHEETS_EXPORT: Required valid service-account json path.'
    assert os.path.exists(
        current_app.config["GOOGLE_SHEETS_EXPORT_SERVICE_ACCOUNT_JSON_PATH"]
    ), 'GOOGLE_SHEETS_EXPORT: Required valid service-account json path.'
    assert isinstance(
        current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"],
        dict,
    ), 'GOOGLE_SHEETS_EXPORT: Required share permissions.'
    if current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"].get('perm_type') == 'anyone':
        assert 'email_address' in \
            current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"] and \
            current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"]['email_address']\
            == None, \
            'GOOGLE_SHEETS_EXPORT: For perm_type == "anyone", email_address must be set \
            to None'
    else:
        assert {"email_address", "perm_type", "role"} <= current_app.config[
            "GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"
        ].keys()


def upload_df_to_new_sheet(name: str, df: pd.DataFrame) -> str:
    assert feature_flag_manager.is_feature_enabled("GOOGLE_SHEETS_EXPORT")

    # Convert any non-serializable types like datetimes to strings:
    df_clone = df.copy()
    object_columns = df_clone.select_dtypes(include=['object']).columns
    df_clone[object_columns] = df_clone[object_columns].astype('string')

    gc = gspread.service_account(
        filename=current_app.config["GOOGLE_SHEETS_EXPORT_SERVICE_ACCOUNT_JSON_PATH"],
    )
    spreadsheet = gc.create(f"{name} {datetime.datetime.utcnow().isoformat()}")
    spreadsheet.sheet1.update(
        range_name="A1",
        values=([df_clone.columns.values.tolist()] + df_clone.values.tolist()),
    )
    spreadsheet.share(**current_app.config["GOOGLE_SHEETS_EXPORT_SHARE_PERMISSIONS"])
    return spreadsheet.id
